/***********************************************************************
 
    c++dbc library. A free extensible C++ database connectivity library
    Copyright (C) 2002-2009  Markus Graf, M.Sc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License only.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>

************************************************************************/
/* 
 * Version information
 *  version 0.3
 *
 * modification history
 *  2002-03-20 gm   reimplementation of database connectivity library as part of expressRM (for more information see http://expressrm.grafs-world.de)
 *  ...        gm   a lot undocumented stuff happened due to several time consuming and occupying tasks as my masters degree a.s.o.
 *  2009-03-21 gm   code-review, readded transaction calls from previous implementation
 */

#if !defined(_IDATABASE_H)
#define _IDATABASE_H

#include "types.h"
class IResultSet;

/**
 * Database Interface (establishes connection to a special database management system and a database)
 * The IDatabase is the interface class from which every Database-Connection is inherited.
 *
 * A general database-connection implementation is done in class Database, to enable common support.
 * For more about the special classes see \code Database \endcode or \code SqLite, MySql \endcode.
 *
 */
class CPPDBC_API IDatabase
{
public:
  /**
   * interface definition for freeResultSet
   * frees result set created by this database connection and removes it from the list of opened results
   * @param prs pointer to the result set created by execQuery of this instance
   * @return false
   * @remark is implemented once in subclass \code Database \endcode to support open list behaviour and auto-deletion if developer forgets to delete the data
   */
  virtual bool freeResultSet(IResultSet* prs) {return false;}
  
  /**
   * interface definition for hasResultSet
   * checks if this database connection has a result set in the open-list of results (that means it was also created by this instance)
   * @param prs pointer to the result set created by execQuery of this instance
   * @return false
   * @remark is implemented once in subclass \code Database \endcode that supports open lists.
   */  
  virtual bool hasResultSet(IResultSet* prs) { return false; }
  
  /**
   * connects to a database \arg database at the \arg host specified with the user-password combination \arg user \arg pwd.
   * @param host the hostname/ip-address of the server or a file name (for file-based database management systems, i.ex. SqLite)
   * @param database database name
   * @param user username to log in
   * @param pwd password to log in
   * @param port optional port if the server runs on another port than its default installation
   * must be implemented in each subclass, as the connection is different for each dbms.
   */
  virtual bool connect(char* host, char* database, char* user, char* pwd, int port=0) {return false;}
  
  /**
   * connects to the database management system at the \arg host specified with the user-password combination \arg user \arg pwd.
   * @param host the hostname/ip-address of the server or a file name (for file-based database management systems, i.ex. SqLite)
   * @param user username to log in
   * @param pwd password to log in
   * @param port optional port if the server runs on another port than its default installation
   * @return false
   * must be implemented in each subclass, as the connection is different for each dbms.
   */  
  virtual bool connect(char* host, char* user, char* pwd, int port=0) {return false;}
  
  /**
   * closes the connection
   * @return false 
   * must be implemented in each subclass, as closing is different for each dbms.
   */
  virtual bool close() {return false;}
  
  /**
   * uses a database
   * @param database name of the database to be used
   * must be implemented in each subclass, as using a database is a dbms-specific task
   */
  virtual void use(char* database) {}

  /**
   * executes a query
   * @param query the sql-query-statement 
   * @return NULL
   * must be implemented
   */
  virtual IResultSet* execQuery(char* query) {return NULL;}

  /**
   * executes an insert statement
   * @param query the sql-statement 
   * @return 0 (or number of rows inserted)
   * must be implemented
   */
  virtual int execInsert(char* query) {return 0;}

  /**
   * executes an update statement
   * @param query the sql-statement 
   * @return 0 (or number of rows updated)
   * must be implemented
   */

  virtual int execUpdate(char* query) {return 0;}

  /**
   * executes a delete statement
   * @param query the sql-statement 
   * @return 0 or number of rows updated
   * must be implemented
   */
  virtual int execDelete(char* query){return 0;}

  /**
   * executes a dml-statement
   * @param query the sql-statement 
   * @return 0 or number of rows deleted
   * must be implemented
   */
  virtual int execDML(char* query){return 0;}
  /**
   * returns the last error
   * @return NULL-Pointer (or const-char*)
   * is implemented once in sublcass \code Database \endcode
   */
  virtual const char* getLastError(){return 0;}

  /**
   * starts a transaction 
   * is implemented in \code Database \endcode, must be overwritten if database management system api provides 
   * functionality that differs from executing the sql-statement "begin transaction"
   */
  virtual void beginTransaction() { }

  /**
   * commits a transaction
   * @return false (or true if done)
   * is implemented in \code Database \endcode, must be overwritten if database management system api provides 
   * functionality that differs from executing the sql-statement "commit"
   */
  virtual bool commit() { return false; }

  /**
   * do a rollback on a transaction
   * @return false (or true if done)
   * is implemented in \code Database \endcode, must be overwritten if database management system api provides 
   * functionality that differs from executing the sql-statement "rollback"
   */
  virtual bool rollback() { return false; }

  /**
   * returns a version string with information about the driver and/or dbms and/or api version.
   * @return version string
   */
  virtual const char* version() { return "none"; }
};

#endif  //_IDATABASE_H
