/***********************************************************************
 
    c++dbc library. A free extensible C++ database connectivity library
    Copyright (C) 2002-2009  Markus Graf, M.Sc.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License only.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>

************************************************************************/

/* IMPORTANT NOTICE:
 *
 * THIS HEADER FILE is the only header-file that must be included per directive in projects that use the dll:
 *
 * #include "c++dbcinc.h"
 *
 *
 * BUT ALL subsequent header files of this document (IDatabase.h, IResultSet.h, IResultSetMetaData.h, types.h) 
 * must be listed in the same directory, so that the compiler can include definitions, classes in your project.
 */

#ifndef __CPPDBC_H__
#define __CPPDBC_H__ __CPPDBC_H__

#include "types.h"
#include "IDatabase.h"
#include "IResultSet.h"
#include "IResultSetMetaData.h"


/**
 * enumeration of supported Database Management Systems that can be created by DatabaseConnectionManager
 */
CPPDBC_API enum DatabaseManagementSystemT { 
  DBMS_SqLite =0
#ifdef DBMS_INCLUDE_MYSQL
  ,DBMS_MySQL =1
#endif
#ifdef DBMS_INCLUDE_ODBC
  ,DBMS_ODBC =2
#endif
#ifdef DBMS_INCLUDE_ODBC3
  ,DBMS_ODBC3=3
#ifdef DBMS_INCLUDE_SQLSERVER2005
  ,DBMS_SQLSERVER2005=4
#endif
#endif

#ifdef DBMS_INCLUDE_NEXT
  ,DBMS_??? =5
#endif
};

const int gDbmsCount = 5;
char* gszDbmsNames[] = {"SqLite", "MySQL", "ODBC", "ODBCv3", "SqlServer2005" };


/**
 * database connection manager, methods are called to generate and remove database connection classes
 * 
 */
class CPPDBC_API DatabaseConnectionManager
{
private:
  /**
   * private constructor, as calls are static up to now
   */
  DatabaseConnectionManager() {}
  /**
   * private destructor, as calls are static up to now
   */
  virtual ~DatabaseConnectionManager() {}

public:
  /** 
   * creates a database interface object for a supported database management system specified by dbms
   * 
   * please free the memory by calling DatabaseConnectionManager::freeDatabaseInterface( &pDatabase );
   *
   * Example:
   * \code
   * IDatabase* pDB = DatabaseConnectionManager::createDatabaseInterface( DBMS_SqLite );
   *
   * // ... do some stuff
   *
   * DatabaseConnectionManager::freeDatabaseInterface( &pDB );
   *
   * \endcode
   */
  static IDatabase* createDatabaseInterface(DatabaseManagementSystemT dbms);


  /** 
   * creates a database interface object for a supported database management system specified by dbmsname
   * 
   * please free the memory by calling DatabaseConnectionManager::freeDatabaseInterface( &pDatabase );
   *
   * Example:
   * \code
   * IDatabase* pDB = DatabaseConnectionManager::createDatabaseInterface( "MySQL" );
   *
   * // ... do some stuff
   *
   * DatabaseConnectionManager::freeDatabaseInterface( &pDB );
   *
   * \endcode
   */
  static IDatabase* createDatabaseInterface(const char* dbmsname);


  /**
   * frees database connection interface object by pointer to a handle
   * @param pdbo reference to the pointer of the object, so that it can be set to NULL
   *
   * after calling this function, the pointer is set to NULL to make clear that this pointer has no longer a reference to an object
   * and thus make it transparent to the developer that called the method.
   * 
   */
  static void freeDatabaseInterface(IDatabase** pdbo);

  /** 
   * returns gpl information to display
   */
  static const char* licenseInformation();

  /**
   * returns gnu gpl warranty information
   */
  static const char* warranty();

  /**
   * returns terms and condition of gnu gpl
   */
  static const char* conditions();

  /**
   * returns current version of this library
   */
  static const char* version();
};


/*
CPPDBC_API IDatabase* createDatabaseObject(DatabaseManagementSystemT dbms);
CPPDBC_API void dropDatabaseObject(IDatabase** pdbo);
*/


#endif // __CPPDBC_H__

